/* 
	Copyright © 2005-2007, Werner Freytag. All rights reserved. 
	http://www.power4mac.com

New in 1.1:
- French localization (thanks to Thomas Didrel!)
- mouse scroll wheel can be used to rotate tokens
- Updated functionality of buttons
- "You've won message"
- Better random pattern generator

New in 1.5:
- Complete rework, numcolumns and numcolors can be edited

New in 1.6:
- Drag & drop possible

New in 1.7
- Better distinguishable colors
- Black&white token theme
- Partial rework of the code

New in 1.7.1
- New Shapes by Steve Ball
- Fix for Leopard
*/

var numrows = 10;

var numcolumns = 4;
var numcolors = 5 ;

var token_theme = 'Default';

var mousedowncolumn = -1;
var dragcolor = -1;

var board = [];
var secretCode = [];

var activeRow = 0;

window.resizeTo( 290, 388 );

if ( window.widget ) {
	if ( widget.preferenceForKey('numcolors') )
		numcolors = widget.preferenceForKey('numcolors');
	if ( widget.preferenceForKey('numcolumns') )
		numcolumns = widget.preferenceForKey('numcolumns');
	if ( widget.preferenceForKey('token_theme') )
		token_theme = widget.preferenceForKey('token_theme');
}

function rotateToken(row, col)
{
	if (row!=activeRow || row>numrows || col>numcolumns)
		return;
	
	var e = window.event;
	var backwards = e.wheelDelta > 1
		|| (e.type && e.type == "contextmenu") || (e.button && e.button == 2) || (e.which && e.which == 3)
		|| e.shiftKey || e.altKey || e.ctrlKey || e.metaKey;
	
	// rotate color
	var color = board[row-1][col-1];
	
	if (backwards) {
		color--;
		if (color < 0 )
			color = numcolors;
	}
	else {
		color++;
		if (color > numcolors)
			color = 0;
	}

	setColor( row, col, color, true );
}

function setColor( row, col, color, updateVar )
{
	var img = document.getElementById( 'token_' + row + '_' + col );
	img.src = 'Images/Tokens/' + token_theme + '/' + color + '.png';
	
	var e = window.event;
	e.stopPropagation();
	e.preventDefault();

	if (updateVar) {
		board[row-1][col-1] = color;
		var div = document.getElementById( 'OkButton' );
		div.setEnabled( allTokensSetInActiveRow() );
		setPointer( div, allTokensSetInActiveRow() );
	}
}

function getColor( row, col )
{
	return board[row-1][col-1];
}

function setPointer( obj, active )
{
	obj.style.cursor = active ? 'pointer' : 'default';
}

function mouseDown( row, col )
{
	mousedowncolumn = activeRow==row ? col : -1;
}

function mouseUp( row, col )
{
	if (activeRow==row && mousedowncolumn==col)
		rotateToken( row, col );
}

function dragStart( row, col )
{
	if ( activeRow>=row && getColor( row, col ) ) {
		
		dragcolor = getColor( row, col );
		mousedowncolumn = -1;
		
		return true;
	}

	return false;
}

function dragOver( row, col )
{
	if (activeRow==row)
		setColor( row, col, dragcolor );
}

function dragLeave( row, col )
{
	if (activeRow==row)
		setColor( row, col, getColor( row, col ) );
}

function drop( row, col )
{
	if (activeRow==row)
		setColor( row, col, dragcolor, true );
}

function mouseOver( img, row )
{
	if (activeRow==row)
		setPointer(img, 1);
}

function mouseOut( img )
{
	setPointer( img, 0);
}

function okClick()
{
	if ( !allTokensSetInActiveRow() )
		return;
	
	activeRow++;
	if (activeRow>numrows)
		discoverCover();
	
	drawBoard();
}

function allTokensSetInActiveRow()
{
	if (!activeRow)
		return;
	
	var allSet = true;
	for ( i=0; i<numcolumns; ++i )
		allSet &= board[activeRow-1][i] > 0;
	return allSet;
}

function newGame()
{
	document.getElementById('message').style.display = 'none';

	document.getElementById('cover').style.display = 'block';
	
	for ( i = 0; i<numcolumns; ++i )
		secretCode[i] = ( (new Date).getMilliseconds() + Math.round( Math.random() * numcolors ) ) % numcolors + 1;
	
	for ( row=0; row<numrows; ++row ) {
		var rowArray = [];
		for ( col=0; col<numcolumns; ++col )
			rowArray[col] = 0;
		board[row] = rowArray;
	}
	
	activeRow = 1;
	
	drawBoard();
}

function setupBoard()
{
	document.getElementById('secretCode').innerHTML = '';
	document.getElementById('trials').innerHTML = '';
	
	// Resize Board
	window.resizeTo( Math.max( 290, 30 * numcolumns + 8 * Math.ceil( numcolumns / 2 ) + 92 ), 388 );
	
	document.getElementById('boardCode').style.width = 30 * numcolumns + 'px';
	document.getElementById('boardRating').style.width = 8 * Math.ceil( numcolumns / 2 ) + 'px';
	
	document.getElementById('cover_left').style.width = 15 * numcolumns - 18 + 'px';
	document.getElementById('cover_right').style.width = 15 * numcolumns - 18 + 'px';
	
	var div = document.getElementById('NewGameButton');
	div.innerHTML = "";
	ImageButton( div, getLocalizedString('New Game'), newGame, numcolumns * 30 - 6, 19);
	
	div = document.getElementById('OkButton');
	div.style.left = 30 * numcolumns + 64 + 'px';
	div.innerHTML = "";
	ImageButton( div, 'ok', okClick, 8 * Math.ceil( numcolumns / 2 ), 18);
	div.setEnabled( false );
	
	var left = 30 * numcolumns + 8 * Math.ceil( numcolumns / 2 ) + 60 + 'px';
	
	div = document.getElementById('flip');
	div.style.left = left;
	
	div = document.getElementById('fliprollie');
	div.style.left = left;

	// Board
	var innerHTML = '';
	innerHTML += '<table cellpadding="0" cellspacing="0">';
	innerHTML += '<tr>';
	for ( col=1; col<=numcolumns; ++col )
		innerHTML += '<td class="token"><img src="Images/Tokens/' + token_theme + '/0.png"/></td>';
	innerHTML += '</tr>';
	innerHTML +='</table>';
	document.getElementById('secretCode').innerHTML = innerHTML;
	
	var innerHTML = ''
	innerHTML += '<table cellpadding="0" cellspacing="0">';
	for ( row=numrows; row>0; --row ) {
		innerHTML += '<tr id="row_' + row + '">';
		innerHTML += '<td class="number">' + row + '</td>';
		for ( col=1; col<=numcolumns; ++col )
			innerHTML += '<td class="token"><img src="Images/Tokens/' + token_theme + '/0.png" id="token_' + row + '_' + col + '"'
				 + ' oncontextmenu="rotateToken(' + row + ',' + col + ')"'
				 + ' onmousewheel="rotateToken(' + row + ',' + col + ')"'
				 + ' onmousedown="mouseDown(' + row + ',' + col + ')"'
				 + ' onmouseup="mouseUp(' + row + ',' + col + ')"'
				 + ' ondragstart="return dragStart(' + row + ',' + col + ')"'
				 + ' ondragover="dragOver(' + row + ',' + col + ')"'
				 + ' ondragleave="dragLeave(' + row + ',' + col + ')"'
				 + ' ondrop="drop(' + row + ',' + col + ')"'
				 + ' onmouseover="mouseOver(this, ' + row + ')"'
				 + ' onmouseout="mouseOut(this)"'
				 + ' style="-khtml-user-drag:element;"'
				 + '/></td>';
		innerHTML += '<td class="rating" align="center">';
		for ( y=0; y<2; ++y ) {
			innerHTML += '<table cellpadding="0" cellspacing="0"><tr>';
			for ( x=0; x < Math.ceil( numcolumns / 2 ); ++x ) {
				if ( y * Math.ceil( numcolumns / 2 ) + x < numcolumns ) {
					innerHTML += '<td>';
					innerHTML += '<img src="Images/PinNone.png" class="pin" name="pin"/>';
					innerHTML += '</td>';
				}
			}
			innerHTML +='</tr></table>';
		}
		innerHTML += '</td>';
		innerHTML += '</tr>';
	}
	innerHTML += '</table>';
	document.getElementById('trials').innerHTML = innerHTML;

	// Back
	document.getElementById('back').style.left = 15 * numcolumns + 4 * Math.ceil( numcolumns / 2 ) - 66 + 'px';
    
    new AppleGlassButton(
            document.getElementById("doneButton"),
            getLocalizedString('Done'),
            hideBack);
            
	newGame();
}

function drawBoard()
{
	var imgs = document.getElementById('secretCode').getElementsByTagName('img');
	var pins = document.getElementsByName('pin');
	
	// display board pins
	for ( col = 0; col<numcolumns; ++col )
		imgs[col].src = 'Images/Tokens/' + token_theme + '/' + secretCode[col] + '.png';
	
	// Pins
	for ( row = numrows; row>0; --row ) {
		
		var theRow = document.getElementById('row_' + row);
		
		// display board pins
		for ( col = 1; col<=numcolumns; ++col ) {
			var img = document.getElementById( 'token_' + row + '_' + col );
			img.src = img.src = 'Images/Tokens/' + token_theme + '/' + board[row-1][col-1] + '.png';
			img.style['-apple-dashboard-region'] = row<=activeRow ? 'dashboard-region(control circle 0 0 0 0)' : '';
		}
	}

	// Rating
	for ( row = numrows; row>0; --row ) {
		
		var theRow = document.getElementById('row_' + row);

		var blacks = 0;
		var whites = 0;
		
		var secretRow = [];
		var trialRow = [];
		
		// rate only when all columns are filled
		var allSet = true;
		for ( i=0; i<numcolumns; ++i )
			allSet &= board[row-1][i] > 0;

		if (allSet) {
			for ( col = 0; col<numcolumns; ++col ) {
				
				trialColor = board[row-1][col];
				secretColor = secretCode[col];
				
				if (trialColor==secretColor)
					blacks++;
				
				else {
					secretRow.push( secretColor );
					trialRow.push( trialColor );
				}
			}
			
			for ( i=0; i<secretRow.length; ++i ) {
				for ( j=0; j<trialRow.length; ++j ) {
					if (secretRow[i]!=false && secretRow[i]==trialRow[j]) {
						whites++;
						secretRow[i] = false;
						trialRow[j] = false;
						break;
					}
				}
			}
			
			// Gewonnen!
			if ( blacks==numcolumns ) {
	
				discoverCover();
				
				var message = document.getElementById( 'message' );
				
				message.innerHTML = getLocalizedString( 'Congratulations!<br>You\'ve found the secret code!' );
				message.style.width = 30 * numcolumns + 8 * Math.ceil( numcolumns / 2 ) + 45 + 'px';
				message.style.display = 'block';
			}
		}
		
		// display rating
		for ( i = 0; i < numcolumns; ++i ) {
			if (blacks>0) {
				pins[(10-row) * numcolumns + i].src = 'Images/PinBlack.png';
				blacks--;
			}
			else if (whites>0) {
				pins[(10-row) * numcolumns + i].src = 'Images/PinWhite.png';
				whites--;
			}
			else {
				pins[(10-row) * numcolumns + i].src = 'Images/PinNone.png';
			}
		}
	}
	
	// Numbers
	for ( row = numrows; row>0; --row ) {
		var theRow = document.getElementById('row_' + row);
		var number = theRow.getElementsByTagName('td')[0];
		number.className = row==activeRow ? 'number active' : 'number';
	}
	
	var div = document.getElementById( 'OkButton' );
	div.setEnabled( allTokensSetInActiveRow() );
	div.style.top = ( 54 + ( 10 - activeRow ) * 30 ) + 'px';
	div.style.display = activeRow ? 'block' : 'none';
}

function discoverCover()
{
	document.getElementById('cover').style.display = 'none';
	activeRow = 0;
}

function updatePrefs()
{
	var new_numcolumns = document.getElementsByName('numcolumns')[0].value;
	var new_numcolors = document.getElementsByName('numcolors')[0].value;
	var new_token_theme = document.getElementsByName('token_theme')[0].value;
	
	if ( new_numcolumns==numcolumns && new_numcolors==numcolors && new_token_theme == token_theme )
		return;
	
	numcolumns = new_numcolumns;
	numcolors = new_numcolors;
	token_theme = new_token_theme;
	
	if (window.widget) {
		widget.setPreferenceForKey( numcolors, 'numcolors' );
		widget.setPreferenceForKey( numcolumns, 'numcolumns' );
		widget.setPreferenceForKey( token_theme, 'token_theme' );
	}
	
	setupBoard();
}

function getLocalizedString (key)
{
	try {
		var ret = localizedStrings[key];
		if (ret === undefined)
			ret = key;
		return ret;
	} catch (ex) {}

	return key;
}

function showBack()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	widget.prepareForTransition("ToBack");
	
	front.style.display="none";
	back.style.display="block";
		
	if (window.widget) {
		setTimeout ('widget.performTransition();', 0);  
	}
	
	document.getElementById('fliprollie').style.display = 'none';
}

function hideBack()
{
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	widget.prepareForTransition("ToFront");
				
	back.style.display="none";
	front.style.display="block";
	
	updatePrefs();
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
}

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};
function mousemove (event)
{
	if (!flipShown)
	{
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
				
		var starttime = (new Date).getTime() - 13;
				
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the info button
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
				
		var starttime = (new Date).getTime() - 13;
				
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
				
		
	T = limit_3(time-animation.starttime, 0, animation.duration);
		
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
		
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c)
{
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease)
{
	return from + (to - from) * ease;
}

function enterflip(event)
{
	document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event)
{
	document.getElementById('fliprollie').style.display = 'none';
}